# PAX - the Pythonic API for XML
# developed by Lalo Martins <lalo@laranja.org>
# this file is placed on public domain

import core
from HTMLParser import HTMLParser
from backwards_compatibility import *

class Document(core.Container):
    def __repr__(self):
        return u'<PAX Document: %s>' % self.children

# get a namespace, if provided
def split_ns(spec, default=u''):
    if ':' in spec:
        return spec.split(':', 1)
    else:
        return [default, spec]

class _parse_in_progress(HTMLParser):
    def __init__(self):
        self.result = Document()
        self.stack = [self.result]
        self.namespaces = []
        self.ns_pushed = 0
        self.ns_popped = 0
        HTMLParser.__init__(self)

    def __raw(self):
        return self.get_starttag_text()

    def ns(self, prefix):
        for layer in self.namespaces:
            if layer.has_key(prefix):
                return layer[prefix]
        return ''

    def handle_starttag(self, rawname, raw_attributes):
        ns, name = split_ns(rawname)
        attributes = {}
        for aspec, avalue in raw_attributes:
            if aspec == u'xmlns':
                self.add_namespace(None, avalue)
            if aspec[:6] == u'xmlns:':
                self.add_namespace(aspec[6:], avalue)
        for aspec, avalue in raw_attributes:
            if aspec[:5] == u'xmlns':
                continue
            ans, aname = split_ns(aspec, ns)
            ans = self.ns(ans)
            attributes.setdefault(ans, {})[aname] = avalue
        if self.ns_pushed:
            nsdecls = self.namespaces[0]
        else:
            nsdecls = {}
        ns = self.ns(ns)
        element = core.Element(ns, name, attributes, nsdecls, self.__raw())
        element.rawname = rawname
        self.stack.append(element)
        self.ns_pushed = 0

    def handle_endtag(self, name):
        topname = getattr(self.stack[-1], 'rawname', None)
        if len(self.stack) > 1:
            self.stack[-2].append(self.stack[-1])
            self.stack.pop()
        self.ns_popped = 0
        if name != topname and len(self.stack):
            #print "minimizing:", name, topname
            self.handle_endtag(name)

    def close(self):
        while len(self.stack) > 1:
            self.stack[-2].append(self.stack[-1])
            self.stack.pop()
        HTMLParser.close(self)

    def handle_decl(self, data):
        self.stack[-1].append(core.Declaration(data))

    def handle_data(self, data):
        self.stack[-1].append(data)

    def handle_charref(self, data):
        self.stack[-1].append(core.CharRef(int(data)))

    def handle_entityref(self, data):
        self.stack[-1].append(core.EntityRef(data))

    def add_namespace(self, prefix, url):
        if self.ns_pushed:
            d = self.namespaces[0]
        else:
            self.ns_pushed = 1
            d = {}
            self.namespaces.insert(0, d)
        d[prefix] = url

    def pop_namespace(self, prefix):
        if not self.ns_popped:
            del self.namespaces[0]
        self.ns_popped = 1
    
    def handle_comment(self, data):
        self.stack[-1].append(core.Comment(data))

def parse(data, builtin_namespaces=None, encoding=None):
    if encoding is None:
        # FIXME: can we get the encoding from the HTML?
        encoding = 'latin1'
    if type(data) is type(''):
        data = unicode(data, encoding)
    parser = _parse_in_progress()
    if builtin_namespaces is not None:
        parser.namespaces.append(builtin_namespaces)
    parser.feed(data)
    parser.close()
    return parser.result
