# PAX - the Pythonic API for XML
# developed by Lalo Martins <lalo@laranja.org>
# this file is placed on public domain

from backwards_compatibility import *

def pack_list(self):
    "remove Nones, expand tuples"
    result = []
    for child in self:
        if child is None:
            continue
        if type(child) in [type(()), type([])]:
            result.extend(child)
        else:
            result.append(child)
    return result

class StopTransform(Exception):
    def __init__(self, result=None, *args):
        Exception.__init__(self, *args)
        self.result = result

class Engine(object):
    def __init__(self):
        self.handlers = ()

    def add_handler(self, ns, handler):
        self.handlers += ((ns, hash(ns), handler),)

    def initialize(self, context):
        context.tr_engine = self
        for ns, h, handler in self.handlers:
            try:
                handler.initialize(context)
            except (AttributeError, TypeError):
                pass

    def transform(self, element, context=None):
        try:
            result = element.copy()
        except AttributeError:
            return element
        try:
            __traceback_info__ = element.raw
        except AttributeError:
            pass
        processed = []
        try:
            e_ns_h = hash(element.ns)
        except AttributeError:
            e_ns_h = None
        try:
            for ns, h, handler in self.handlers:
                if not hasattr(result, 'attributes'):
                    break
                # We compare the hash of the ns against the hash of the element's ns
                # this avoids an expensive unicode character-by-character comparison,
                # in exchange for hash computation which is cached.
                #
                # However, this also means you can't change an element's ns in one
                # transform handler and expect it to affect future handlers.
                #
                # Then again, if you wanted to do that, you probably should be shot.
                #
                #
                #
                #
                # (just kidding)
                if result.attributes.has_key(ns) or e_ns_h == h:
                    processed.append(handler)
                    result = handler(result, context)
        except StopTransform, data:
            result = data.result
        else:
            if not hasattr(element, 'join'):
                for index, child in enumerate(result):
                    result[index] = self.transform(child, context)
        try:
            result.pack()
        except AttributeError:
            try:
                result = pack_list(result)
            except:
                pass
        for handler in processed:
            try:
                pp = handler.postprocess
            except AttributeError:
                pass
            if not callable(pp): continue
            try:
                result = pp(result, context)
            except StopTransform, data:
                return data.result
        return result


class AttributeHandler(object):
    "Handler that handles each attribute in turn"
    def __init__(self, name=u'', ns=u''):
        self.name = name
        self.ns = ns
        self.handlers = ()
        self.preprocess = None
        self.postprocess = None
        self.initialize = None

    def add_handler(self, name, handler):
        self.handlers += ((name, handler),)

    def __call__(self, element, context):
        result = element
        if self.preprocess is not None:
            result = self.preprocess(result, context)
        for name, handler in self.handlers:
            try:
                value = result.attributes[self.ns][name]
            except KeyError:
                continue
            result = handler(result, context, value)
        return result
