# PAX - the Pythonic API for XML
# developed by Lalo Martins <lalo@laranja.org>
# this file is placed on public domain

from pax import *

# An engine that transforms xml like:
# <foo filter:method="upper">xyz</foo>
# by running character data trough getattr(s, name)() -
# in this case, name being "upper" and resulting in:
# <foo filter:method="upper">XYZ</foo>
# where xmlns:filter="http://www.laranja.org/xml/fake/filter"

filter_ns = 'http://www.laranja.org/xml/fake/filter'

if type(str) is type:
    stringtypes = (unicode, str)
else:
    import Types
    stringtypes = (types.StringType, types.UnicodeType)

def filter_handler(element, context):
    name = element.attributes[filter_ns].get('method', None)
    if name is None:
        return element
    for index, child in enumerate(element):
        if isinstance(child, stringtypes):
            element[index] = getattr(child, name)()
    return element, False

from paxtransform import Engine
filter_engine = Engine()
filter_engine.add_handler(filter_ns, filter_handler)
    
if __name__ == '__main__':
    import sys
    if len(sys.argv) > 1:
        srcfile = open(sys.argv[1])
    else:
        srcfile = sys.stdin
        sys.stderr.write('Reading source from stdin\n')
    source = xml2pax(srcfile.read(), {'filter':filter_ns})
    print XML(filter_engine.transform(source))
